package termites;

import java.awt.Color;

import termites.agent.Termite;
import termites.agent.Wood;
import fr.lifl.jedi.controllersCore.AbstractControlGUIController;
import fr.lifl.jedi.gui.control.graphicalControl.controller.GraphicalSimulationControlGUIController;
import fr.lifl.jedi.gui.display.colorGrid.ColorGridGUIController;
import fr.lifl.jedi.gui.display.colorGrid.view.AgentDisplayer;
import fr.lifl.jedi.gui.display.colorGrid.view.ColorGridGUI;
import fr.lifl.jedi.gui.display.colorGrid.view.agentDisplay.ColorRectangleDisplayer;
import fr.lifl.jedi.gui.display.colorGrid.view.agentDisplay.StaticColorTriangleDisplayer;
import fr.lifl.jedi.model.Environment;

/**
 * This class is the main class of the simulation.
 * It contains the main method, where the simulation and the views displaying
 * simulation informations are created.
 * 
 * @author
 *	<table>
 *		<tr>
 *			<td>Yoann Kubera</td>
 *		</tr>
 *		<tr>
 *			<td><a href="http://www.lifl.fr/SMAC/">SMAC Team (Systèmes Multi-Agents et Comportement)</a></td>
 *		</tr>
 *		<tr>
 *			<td><a href="http://www.lifl.fr/">LIFL (Laboratoire d'Informatique Fondamentale de Lille)</a></td>
 *		</tr>
 *		<tr>
 *			<td><a href="http://www.univ-lille1.fr/">University of Lille, France</a></td>
 *		</tr>
 *  </table>
 * @version 
 * 	<table>
 *		<tr>
 *			<td>JEDI V 2.3</td>
 *		</tr>
 *		<tr>
 *			<td>Created the 4th Nov. 2009</td>
 *		</tr>
 *	</table>
 */
public class TermitesMain {
	/**
	 * Main method of the simulation.
	 */
	public static void main(String[] args) {
		// Creation of the environment where the simulation takes place.
		Environment e = new Environment(100, 100); // Dimensions of the environment are 100 x 100 units.
		e.setXAxisTorus(true); // The environment is a torus along the x-axis
		e.setYAxisTorus(true); // The environment is a torus along the y-axis
		
		// Creation of the GUI used to launch the simulation.
		// This line creates a GUI where the simulation is controlled by buttons.
		AbstractControlGUIController<?> simulationControlGUI = new GraphicalSimulationControlGUIController();
		
		// Creation of the core of the simulation.
		// It defines how the simulation is initialized, when the simulation
		// is assumed to be finished, and how the simulation is controlled.
		TermitesSimulationCore core = new TermitesSimulationCore(simulationControlGUI, e);

		/* FROM THIS POINT ON, THE SIMULATION CAN BE INITIALIZED AND PERFORMED */
	
		// At this point, no GUI makes possible to display the environment or results.
		// Thus, we choose to use a GUI where the environment is displayed as
		// a square, where agents are represented by colored shapes.
		
		// Creation of the frame that will display the simulation.
		ColorGridGUI gui = new ColorGridGUI(
				// The environment where the simulation takes place
				e,
				// The name of the frame where the simulation is displayed
				"Termites Simulation GUI",
				// The size on screen, in pixels, of a unit of the environment
				5);
		// Set how agents of the Termite agent family are displayed : as a triangle, which dimensions do not depend
		// on the dimensions of the agent.
		AgentDisplayer disp = new StaticColorTriangleDisplayer(
				// The color of the triangle
				Color.DARK_GRAY, 
				// The width, in environment units, of the triangle
				1,
				// The height, in environment units, of the triangle
				2,
				// If the triangle is displayed in the background of the environment. False means it is displayed
				// on the foreground.
				false); 
		gui.getGraphicalModel().setHowToDisplay(Termite.class, disp);
		// Set how agents of the Wood agent family is displayed : as a rectangle, which dimensions correspond to the
		// dimensions of the wood agent in the environment.
		disp = new ColorRectangleDisplayer(
				// The color of the rectangle
				Color.RED,
				// If the rectangle is displayed in the background of the environment. True means it is displayed
				// on the Background.
				true);
		gui.getGraphicalModel().setHowToDisplay(Wood.class, disp);

		// Binds the newly create GUI with the simulation.
		// It ensures that every time a simulation step ends, the GUI will 
		// update the image it displays.
		ColorGridGUIController ctrl = new ColorGridGUIController(gui);
		core.registerGUIControler(ctrl);
	}
}
