package termites.interaction;

import fr.lifl.jedi.model.Agent;
import fr.lifl.jedi.model.Environment;
import fr.lifl.jedi.model.interactionDeclaration.DegenerateInteraction;
import fr.lifl.jedi.util.SimulationProperties;

/**
 * This interaction involves only a source agent.
 * <p>
 * 	It moves the source agent randomly in the environment.
 * </p>
 * <p>
 * 	Random move is achieved by turning the agent from a randomly chosen 
 * 	angle in [-15,15[, and then, if it is possible, moving it forward 
 * 	in 1 unit.
 * </p>
 * <p>
 * 	This interaction can always be performed.
 * </p>
 * 
 * @author
 *	<table>
 *		<tr>
 *			<td>Yoann Kubera</td>
 *		</tr>
 *		<tr>
 *			<td><a href="http://www.lifl.fr/SMAC/">SMAC Team (Systèmes Multi-Agents et Comportement)</a></td>
 *		</tr>
 *		<tr>
 *			<td><a href="http://www.lifl.fr/">LIFL (Laboratoire d'Informatique Fondamentale de Lille)</a></td>
 *		</tr>
 *		<tr>
 *			<td><a href="http://www.univ-lille1.fr/">University of Lille, France</a></td>
 *		</tr>
 *  </table>
 * @version 
 * 	<table>
 *		<tr>
 *			<td>JEDI V 2.3</td>
 *		</tr>
 *		<tr>
 *			<td>Created the 4th Nov. 2009</td>
 *		</tr>
 *	</table>
 */
public class Wander extends DegenerateInteraction {
	/**
	 * This interaction can always be performed.
	 */
	public boolean preconditions(Environment environment, Agent source) {
		return true;
	}

	/**
	 * Not used in this model.
	 */
	public boolean trigger(Environment environment, Agent source) {
		return true;
	}
	
	/**
	 * This method performs the actions related to the Wander interaction.
	 */
	public void perform(Environment environment, Agent source) {
		// Turns the agent from a randomly chosen angle in [-15,15[
		environment.turnLeft(source, SimulationProperties.random() * 30 - 15);
		// If the agent can move forward in 1 unit, it moves.
		boolean b = environment.canMoveForward(source, 1);
		if(b){
			environment.moveForward(source, 1);
		}
	}
}
