package termites.interaction.functions;

import fr.lifl.jedi.model.Agent;

/**
 * This interface represents the source function of the Put interaction.
 * Agents that can be the source of the Put interaction have to implement it.
 * 
 * @author
 *	<table>
 *		<tr>
 *			<td>Yoann Kubera</td>
 *		</tr>
 *		<tr>
 *			<td><a href="http://www.lifl.fr/SMAC/">SMAC Team (Systèmes Multi-Agents et Comportement)</a></td>
 *		</tr>
 *		<tr>
 *			<td><a href="http://www.lifl.fr/">LIFL (Laboratoire d'Informatique Fondamentale de Lille)</a></td>
 *		</tr>
 *		<tr>
 *			<td><a href="http://www.univ-lille1.fr/">University of Lille, France</a></td>
 *		</tr>
 *  </table>
 * @version 
 * 	<table>
 *		<tr>
 *			<td>JEDI V 2.3</td>
 *		</tr>
 *		<tr>
 *			<td>Created the 5 nov. 2009</td>
 *		</tr>
 *	</table>
 */
public interface PutSourceFunction {
	/**
	 * Checks that the source agent is already carrying a resource.
	 * @return <code>true</code> if the agent is already carrying a resource.
	 */
	public boolean ownsAnElement();
	
	/**
	 * Removes the resource carried by the source agent from its inventory, and returns it.
	 * @return the resource carried by the source agent.
	 */
	public Agent retrieveElement();
}
